static char version_string[]="$Id: 2dch.c,v 1.3 1999/10/13 13:14:59 kise Exp $";
/*
   $Date: 1999/10/13 13:14:59 $
   $Revision: 1.3 $
   $Author: kise $

 *  ץ
 *  
 *   : ޷ζ򥵥ץ󥰤
 *   : Ϥʬ
 *  
 *  ŵ : http://netlib.bell-labs.com/netlib/master/readme.html
 *  
 */

/*
 * Ken Clarkson wrote this.  Copyright (c) 1996 by AT&T..
 * Permission to use, copy, modify, and distribute this software for any
 * purpose without fee is hereby granted, provided that this entire notice
 * is included in all copies of any software which is or includes a copy
 * or modification of this software and in all copies of the supporting
 * documentation for such software.
 * THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
 * REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
 * OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
 */


/*
 * two-dimensional convex hull
 * read points from stdin,
 * one point per line, as two numbers separated by whitespace
 * on stdout, points on convex hull in order around hull, given
 * by their numbers in input order
 * the results should be "robust", and not return a wildly wrong hull,
 * despite using floating point
 * works in O(n log n); I think a bit faster than Graham scan;
 * somewhat like Procedure 8.2 in Edelsbrunner's "Algorithms in Combinatorial
 * Geometry".
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include "const.h"
#include "defs.h"
#include "function.h"

Coord ch_points[N_CH_POINTS][2];
Coord *P[N_CH_POINTS+1]; /* an extra position is used */

void set_points(int n, Coordinate x, Coordinate y)
{
  assert(n <= N_CH_POINTS);
	
  ch_points[n][0] = (Coord)x;
  ch_points[n][1] = (Coord)y;
}

int read_points(int chnum)
{
  int n;
	
  for(n=0;n<chnum;n++){
    P[n] = ch_points[n];
  }
  P[n] = ch_points[0];

  return n;
}


float polygon_calc(int m)
{
  int i;
  Coord x1,x2,answer=0;

  if(m < 3){
    return SMALL;
  }
  else if(m%2 == 0){
    /* m ΤȤ */
    x1 = P[0][0];
    x2 = P[1][0];
    for(i=2;i<=(m/2);i++){
      answer += (P[2*i-2][0]-x1)*(P[2*i-1][1]-P[2*i-3][1])+
	        (P[2*i-1][0]-x2)*(P[2*i][1]-P[2*i-2][1]);
    }
  }
  else{
    /* m ΤȤ */
    x1 = P[0][0];
    for(i=2;i<=m;i++){
      answer += (P[i-1][0]-x1)*(P[i][1]-P[i-2][1]);
    }
  }
	
  if(answer < 0){
    /* answer ΤȤ */
    answer = -1 * answer;
  }

  return (float)answer/2.0;
}

int ccw(Coord **P, int i, int j, int k) {
  Coord	a = P[i][0] - P[j][0],
    b = P[i][1] - P[j][1],
    c = P[k][0] - P[j][0],
    d = P[k][1] - P[j][1];
  return a*d - b*c <= 0; /* true if points i, j, k counterclockwise */
}


#define CMPM(c,A,B) \
v = (*(Coord**)A)[c] - (*(Coord**)B)[c];\
if (v>0) return 1;\
if (v<0) return -1;

int cmpl(const void *a, const void *b) {
  double v; 
  CMPM(0,a,b);
  CMPM(1,b,a);
  return 0;
}

int cmph(const void *a, const void *b) {return cmpl(b,a);}


int make_chain(Coord** V, int n, int (*cmp)(const void*, const void*)) {
  int i, j, s = 1;
  Coord* t;

  qsort(V, n, sizeof(Coord*), cmp);
  for (i=2; i<n; i++) {
    for (j=s; j>=1 && ccw(V, i, j, j-1); j--){}
    s = j+1;
    t = V[s]; V[s] = V[i]; V[i] = t;
  }
  return s;
}

int ch2d(int n)  {
  int u = make_chain(P, n, cmpl);		/* make lower hull */
  if (!n) return 0;
  P[n] = P[0];
  return u+make_chain(P+u, n-u+1, cmph);	/* make upper hull */
}

float area_calc(int chull)
{
  return polygon_calc(chull); 
}

Coord distance2(int i, int j)
{
  Coord x, y;

  x = P[i][0] - P[j][0];
  y = P[i][1] - P[j][1];

  return (x*x + y*y);
}

Coord area1(int i, int j)
{
  return (P[i][1] - P[i+1][1])*(P[j][0] - P[j+1][0]) - 
         (P[j][1] - P[j+1][1])*(P[i][0] - P[i+1][0]); 
}

float diameter_calc(int chull)
{
  int i, j, is, js;
  Coord max;

  is = js = 0;

  for(i = 0; i < chull; i++){
    if(P[i][1] > P[is][1]){
      is = i;
    }
    if(P[i][1] < P[js][1]){
      js = i;
    }
  }
	
  max = distance2(is, js);

  i = is;
  j = js;
	
  do{
    if(area1(i, j) < 0.0){
      j = (j + 1) % chull;
    }
    else{
      i = (i + 1) % chull;
    }
		
    if(distance2(i, j) > max){
      max = distance2(i, j);
    }
  }while((i != is) || (j != js));

  return (float)sqrt(max);
}

/*
 *  get_vertex
 *   i ܤĺκɸ֤ؿ
 *  returns i-th points on a convex-hull
 */
void get_vertex(Point *tmp, int i)
{
  tmp->x = (float)P[i][0];
  tmp->y = (float)P[i][1];	
}
