// VoronoiLibTest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

extern "C"
{
	#include "voronoi.c"
}

#include "RegionRep.h"
#include <algorithm>
#include <math.h>

#define MAX_EDGES 10					//Max. no. of edges at a vertex
#define PI 3.1415926535

using namespace std;


typedef struct segEnd{
	int x;
	int y;
	bool end;							//0 indicates its 1st end of line segment, 1 => second end
	int lineIndex;						//stores line index to which it belongs to
	int count;							//every line segment is a part of exactly 2 regions
	int edgesWith;						//id of the line with which this point edges with. 
										//Every (end) point of a line - which is a vertex - can continue to only 2
										//edges (of different regions). -1 indicates it hasnt been traversed with
										//any possible edge, x indicates it has been traversed with x edge
}SegEnd;

void alignVoronoiEdges(string voronoiEdgeFile, vector <CRegionRep> & vRegions, const int &height, const int &width);
bool sortByXthenY(SegEnd p1, SegEnd p2);
int findAdjacentEdge(int a, int b, vector <SegEnd> & pointsTable, int ** &pIndex, char &turn, const int &height, const int &width, bool &isVertex);
float findAngle(int a, int b, int c, vector <SegEnd> &pointsTable, int ** &pIndex);
void insertBEdges(vector <int> &edgePts, int x, int y, int &noOfLines, vector <SegEnd> &pointsTable, const int &height, const int &width);
void printRegions(vector <CRegionRep> & regions, string tifFile, const int &width, const int &height);
void mergeRegions(vector <CRegionRep> & vRegions);


int _tmain(int argc, _TCHAR* argv[])
{
    ifstream inFile;
	string tifFile;					//tiff input file
	string outFile;					//file containing voronoi segments for corresponding tiff input file
	char **arguments;
	int height, width;				//height and width of the image
	ImageData imgd;					//image data (as per lib datastructure)
	BlackPixel 	*bpx;				//blackpixel datastructure
	TIFF *image;

	vector <CRegionRep> vRegions;		//vector of voronoi regions
	vector <CRegionRep> pRegions;		//vector of approximated polygonal regions

	if(argc <= 2)
	{
		printf("Please see the usage\n");
		return(0);
	}


	tifFile = argv[1];

	try{

		//copying commandline arguments
		arguments = new char* [argc+1];
		arguments[0] = new char[strlen(argv[0])];
		strcpy(arguments[0], argv[0]);

		//inserting input file name and output filename	
		
		cout << "Starting File " << tifFile << endl;
		outFile = tifFile;
		outFile.replace(outFile.size()-4, 4, ".txt");

		arguments[1] = new char[tifFile.size()];
		strcpy(arguments[1], tifFile.c_str());
		//arguments[1] = file1;
		arguments[2] = new char[outFile.size()];
		strcpy(arguments[2], outFile.c_str());
		//arguments[2] = file2;
				
		//copying rest of the arguments
		for(int i = 2; i < argc; i++)
		{
			arguments[i+1] = new char[strlen(argv[i])];
			strcpy(arguments[i+1], argv[i]);
		}
		arguments[argc+1] = '\0';

		//Get Voronoi tessellation
		getVoronoiSegments(arguments);


		//freeing arguments
		for(int i = 1; i < argc+1; i++)
		{
//				delete [] arguments[i];
		}
//			delete [] arguments;

		//Get Image height and width
        // Open the TIFF image
		//if((image = TIFFOpen(tifFile.c_str(), "r")) == NULL){
		//	fprintf(stderr, "Could not open incoming image\n");
		//	exit(42);
		//}

		//TIFFGetField(image, TIFFTAG_IMAGEWIDTH, &width);
		//TIFFGetField(image, TIFFTAG_IMAGELENGTH, &height);

		width = imgWidth;
		height = imgHeight;

		//Getting zone-wise representation of voronoi cells
		vRegions.clear();
		alignVoronoiEdges(outFile, vRegions, height, width);

		//Merging small polygons into bigger ones
		mergeRegions(vRegions);

		//Print regioins to a file
		printRegions(vRegions, tifFile, width, height);

		//Approximate Polygons
		//approximatePolygons(vRegions, pRegions);

	}
		//Catch any exception derived from the standard C++ exception
	catch(std::exception &ex){
//			 Get the error message
//			 and print to screen
		cout << ex.what();  
	}
//		 Catch all unknown excpetions
	catch(...)
	{
		cout << "Unknown exception!!\n";
	}
	
	//free(bpx);
	free(neighbor);
    free(area);
    free(sites);
    free(lineseg);
    free(endp);
//    free(imgd1.image);

	return 0;
}


/*! regionFormation function lines up voronoi segments to form voronoi regions
\param inFile a file containing the list of voronoi segments (x1 x2 y1 y2) arranged randomly
\param vector of regions with their corresponding segments
*/

void alignVoronoiEdges(string randomEdgeFile, vector <CRegionRep> & vRegions, const int &height, const int &width)
{
	ifstream inFile;
	string line;							//line containing x1 x2 y1 y2 from inFile
	int x1, x2, y1, y2;
	//vector <DLLineShape> lineSegs;			//vector storing line segments from the input file
	int **pIndex;							//stores the indices of sorted points of each line segment
	int noOfLines;
	int top;								//pointer to the pointsTable to first 'non-zero count' first (end = 0) point 
	int a, b;								// a---b   where x---y line seg
	char turn;								//'u': unknown,  'l': left,   'r': right
	vector <int> leftBPts, rightBPts, topBPts, bottomBPts;	//vectors to store boundary voronoi points of the document
	int bPt;								//boundary Point
	bool isVertex;


	vector <SegEnd> pointsTable;			//vector to hold points table of indices
	
	try{

		inFile.open(randomEdgeFile.c_str());

		noOfLines = 0;
		//Reading the file
		while(inFile)
		{
			x1 = -1;
			inFile >> x1 >> x2 >> y1 >> y2;
			if (x1 == -1)
				continue;
			// cout << x1 << " " << x2 << " " << y1 << " " << y2 << endl;
			//DLLineShape temp(x1, y1, x2, y2);
			//lineSegs.push_back(temp);

			//Pushing in boundary points
			if(x1 == 0)
				leftBPts.push_back(y1);
			else if(x2 == 0)
				leftBPts.push_back(y2);
			else if(y1 == 0)
				topBPts.push_back(x1);
			else if(y2 == 0)
				topBPts.push_back(x2);
			else if(x1 == width-1)
			{
				rightBPts.push_back(y1);
				//cout << "Pushing for x1 match " << x1 << " " << y1 << " " << x2 << " " << y2 << endl;
			}
			else if(x2 == width-1)
			{
				rightBPts.push_back(y2);
				//cout << "Pushing for x2 match " << x1 << " " << y1 << " " << x2 << " " << y2 << endl;
			}
			else if(y1 == height-1)
				bottomBPts.push_back(x1);
			else if(y2 == height-1)
				bottomBPts.push_back(x2);

			
			SegEnd pt;
			//First end of line segment
			pt.x = x1;
			pt.y = y1;
			pt.end = 0;
			pt.lineIndex = noOfLines;
			pt.count = 2;
			pt.edgesWith = -1;
			pointsTable.push_back(pt);

			//Second end of line segment
			pt.x = x2;
			pt.y = y2;
			pt.end = 1;
			pt.lineIndex = noOfLines;
			pt.count = 2;
			pt.edgesWith = -1;

			pointsTable.push_back(pt);
			noOfLines++;
		}
		inFile.close();

		//Sorting boundary points
		insertBEdges(leftBPts, 0, -1, noOfLines, pointsTable, height, width);
		insertBEdges(topBPts, -1, 0, noOfLines, pointsTable, height, width);
		insertBEdges(rightBPts, width-1, -1, noOfLines, pointsTable, height, width);
		insertBEdges(bottomBPts, -1, height-1, noOfLines, pointsTable, height, width);

		//Sorting
		std::sort(pointsTable.begin(), pointsTable.end(), sortByXthenY);

		//printing points
		/* for(int i = 0; i < pointsTable.size(); i++)
		{
			cout << pointsTable[i].x << " " << pointsTable[i].y << " " << pointsTable[i].count << " " << pointsTable[i].edgesWith << endl;
		}
		*/

		//Creating arrays to hold points indices
		pIndex = new int*[noOfLines];
		if(pIndex != NULL)
		{
			for(int i = 0; i < noOfLines; i++)
				pIndex[i] = new int[2];
		}

		//Updating point indices
		for(int i = 0; i < pointsTable.size(); i++)
		{
			pIndex[pointsTable[i].lineIndex][pointsTable[i].end] = i;
		}

		//Creating regions begin
		top = 0;
		while(top < pointsTable.size())			
		{
			//each 'while' pass builds up 1 region
			CRegionRep reg;

			//reaching a non-covered edge's first point
			//[an edge is considered to be covered if it has been accounted for 2 regions]
			while(pointsTable[top].end == 1 || pointsTable[top].count <= 0)
				top++;

			a = top;
			turn = 'u';
			//creating noOfRegions-th region
			while(1)
			{
				pointsTable[a].count--;												//decreasing the count
				b = pIndex[pointsTable[a].lineIndex][(pointsTable[a].end + 1)%2];
				pointsTable[b].count--;												//decreasing the count

				DLLineShape ls(pointsTable[a].x, pointsTable[a].y, pointsTable[b].x, pointsTable[b].y);
				//pushing an edge;			
				reg.addEdge(ls);
				//cout << "Adding Edge: " << ls.dlGetFirstPoint().x << "\t" << ls.dlGetFirstPoint().y << "\t\t" << ls.dlGetSecondPoint().x << "\t" << ls.dlGetSecondPoint().y << endl;

				//Checking if a circle is completed
				if(pointsTable[top].x == pointsTable[b].x && pointsTable[top].y == pointsTable[b].y)
				{
					//region completed
					break;
				}

				a = findAdjacentEdge(a,b, pointsTable, pIndex, turn, height, width, isVertex);
				if(a == -1)
				{
					cout << "Incomplete region found!" << endl;
					break;
				}
				if(isVertex)
				{
					pointsTable[b].edgesWith = a;
                    pointsTable[a].edgesWith = b;
				}
							
			}

			//Pushing one completed region
			vRegions.push_back(reg);
			cout << "Region Finished.." << endl;

		}
	}
	catch(std::exception &ex){
		// Get the error message
		// and print to screen
		cout << ex.what();  
	}
	// Catch all unknown excpetions
	catch(...)
	{
		cout << "Unknown exception!!\n";
	}


}

/*! sortByXthenY function sorts two points based on X and then Y
\param p1 1st point
\param p2 2nd point (see Points structure)
\return true if p1 < p2 else false
*/
bool sortByXthenY(SegEnd p1, SegEnd p2)
{
	if(p1.x < p2.x)
		return true;
	else if(p1.x > p2.x)
		return false;
	else if(p1.y < p2.y)
		return true;
	else
		return false;
}

/*! findAdjacentEdge finds next edge given an edge
It does the following function:
1. connects if a--b was edging with document edges
2. takes a decision at a vertex (whether to go left or right)
\param a starting pt of edge
\param b ending pt of edge
\param pointsTable
\param pIndex stores the indices of sorted points for each line segment
\param turn to be taken at each vertex
\return index of next starting point
*/
int findAdjacentEdge(int a, int b, vector <SegEnd> &pointsTable, int ** &pIndex, char &turn, const int &height, const int &width, bool &isVertex)
{
	int up, down;
	int leftIndex, rightIndex;			//index to hold left and right edges
	float minAngle, maxAngle;
	int indexArray[MAX_EDGES];
	float angleArray[MAX_EDGES];
	int countEdges;
	int nextPt;
	int reduceEdges = 0;				//sometimes faulty edges appear as part of U-turn, which make a straight line a vertex
										//A---->B----->C
										// <----
										//hence B appears like a vertex, but its not

	isVertex = false;
	
	up = b;
	down = b;
	
	/*! Cases depending on how many edges follow the current a--b one:
	#1 Only 1 edge - i.e. no vertex
	#2 Only 2 edges - a vertex - decision to be made is ONLY which one to take among them
		#a. If turn = 'u', choose any and update turn
		#b. If turn = 'l' or 'r', continue on that
	#3 >2 edeges - a vertex - decision to be made is which 2 are of interest and then apply #2
	*/
	

	countEdges = 0;
	do{
		int temp1 = pointsTable[b].x;
		int temp2 = pointsTable[b].y;

		if(pointsTable[up-1].x == pointsTable[b].x && pointsTable[up-1].y == pointsTable[b].y) // && pointsTable[up-1].count > 0) // && pointsTable[b].edgesWith != up-1)	// vertex
		{
			indexArray[countEdges] = up-1;
            countEdges++;
			up--;
		}
		else if(pointsTable[down+1].x == pointsTable[b].x && pointsTable[down+1].y == pointsTable[b].y) // && pointsTable[down+1].count > 0) // && pointsTable[b].edgesWith != down+1)	// vertex
		{
			indexArray[countEdges] = down+1;
			countEdges++;
			down++;
		}
		else
			break;
	}while(1);

	switch(countEdges)
	{

	case 1: //case #1
		nextPt = indexArray[0];
		break;
	case 2:	//case #2 n #3
	case 3:
		nextPt = -1;
		isVertex = true;
		minAngle = 9999;
		maxAngle = -9999;
		for(int i = 0; i < countEdges; i++)
		{
			angleArray[i] = findAngle(a,b,indexArray[i], pointsTable, pIndex);

			if((int)(angleArray[i]*100) == -314 || (int)(angleArray[i]*100) == 314)
			{
				reduceEdges++;
				continue;
			}
			if(angleArray[i] < minAngle)
			{
				minAngle = angleArray[i];
				rightIndex = indexArray[i];
			}
			if(angleArray[i] > maxAngle)
			{
				maxAngle = angleArray[i];
				leftIndex = indexArray[i];
			}

		}

		if(turn != 'r' && pointsTable[leftIndex].count > 0 && pointsTable[b].edgesWith != leftIndex )		// 0 is left
		{
			turn = 'l';
			nextPt = leftIndex;
		}
		else if(pointsTable[rightIndex].count > 0 && pointsTable[b].edgesWith != rightIndex)
		{
			turn = 'r';
			nextPt = rightIndex;
		}

		//swap l/r if boundary edge is continuing in boundary at a vertex (will happen only in start-case)
		if((pointsTable[a].x == 0 || pointsTable[a].x == width-1 || pointsTable[a].y == 0 || pointsTable[a].y == height-1) 
			&& minAngle == 0)
		{
			turn = 'l';
			nextPt = leftIndex;
		}
		else if((pointsTable[a].x == 0 || pointsTable[a].x == width-1 || pointsTable[a].y == 0 || pointsTable[a].y == height-1) 
			&& maxAngle == 0)
		{
			turn = 'r';
			nextPt = rightIndex;
		}

		if(countEdges-reduceEdges > 1)
		{
            cout << "Taking a turn " << turn << endl;
		}
		else
		{
			cout << "Imaginary turn removed" << endl;
			isVertex = false;
		}
		
		break;
	default: 
		printf("Dangling Edge!\n");
		nextPt = -1;
		break;

	}
	return nextPt;
}

/*! findAngle finds the angle between a-->b and c-->d vectors between -180 to 180 (in radians)
in anti-clockwise direction starting from a-->b to c-->d
\param a
\param b
\param c
\param pointsTable to calculate d
\pIndex to calculate d
\return angle in radians
*/
float findAngle(int a, int b, int c, vector <SegEnd> &pointsTable, int ** &pIndex)
{
	int x1, x2, y1, y2;
	int d;
	float result;

	d = pIndex[pointsTable[c].lineIndex][(pointsTable[c].end + 1)%2];

	x1 = pointsTable[b].x - pointsTable[a].x ;
	y1 = pointsTable[b].y - pointsTable[a].y;

	x2 = pointsTable[d].x - pointsTable[c].x;
	y2 = pointsTable[d].y - pointsTable[c].y;

	if((x1 == 0 && y1 == 0) || (x2 == 0 || y2 ==0))
	{
		cout << "A dot line is detected" << endl;
	}

	result = atan2((float)(x1*y2-x2*y1),(float)(x1*x2+y1*y2));

	//Since  co-ordinate system of images have y-axis reversed, we should reverse the sign of result
	//in order to get desired effect
	result = result*-1;

	//if(result < 0)
	//	result += 2*PI;

	return result;

}

/*! insertBEdges creates edges at document boundaries
\param edgePts list of edges on document boundaries
\param x abscissa
\param y ordinates -1 implies changing co-ordinate and hence should be read from edgePts
\param noOfLines read so far..
\param pointsTable
*/
void insertBEdges(vector <int> &edgePts, int x, int y, int &noOfLines, vector <SegEnd> &pointsTable, const int &height, const int &width)
{
	std::sort(edgePts.begin(), edgePts.end());

	SegEnd pt;
	//First end of line segment
	if(y == -1)			//either left or right
	{
        pt.x = x;
        pt.y = 0;
	}
	else
	{
		pt.x = 0;
		pt.y = y;
	}
	pt.end = 0;
	pt.lineIndex = noOfLines;
	pt.count = 1;
	pt.edgesWith = -1;
	pointsTable.push_back(pt);

	//cout << "\nAdded " << pt.x << "," << pt.y << endl;

	for(int i = 0; i < edgePts.size(); i++)
	{
		//end point of last line
		if(y == -1)			//either left or right
		{
			pt.x = x;
			pt.y = edgePts[i];
		}
		else				//either top or bottom
		{
			pt.x = edgePts[i];
			pt.y = y;
		}
		pt.end = 1;
		pt.lineIndex = noOfLines;
		pt.count = 1;
		pt.edgesWith = -1;
		pointsTable.push_back(pt);

		//cout << "Added " << pt.x << "," << pt.y << endl;
		
		noOfLines++;

		// first point of next line
		pt.end = 0;
		pt.lineIndex = noOfLines;
		pointsTable.push_back(pt);

		//cout << "\nAdded " << pt.x << "," << pt.y << endl;

	}


	//end point of last line
	if(y == -1)			//either left or right
	{
        pt.x = x;
        pt.y = height-1;
	}
	else				//either top or bottom
	{
		pt.x = width-1;
		pt.y = y;
	}
	pt.end = 1;
	pt.lineIndex = noOfLines;
	pt.count = 1;
	pt.edgesWith = -1;
	pointsTable.push_back(pt);

	cout << "Added " << pt.x << "," << pt.y << endl;

	noOfLines++;


}

/* printRegions prints the polygons in GEDI format
\param regions to be printed
\param tifFile
\return void
*/
void printRegions(vector <CRegionRep> & regions, string tifFile, const int &width, const int &height)
{
	ofstream outFile;
	string xmlFile;
	
	//Copying location of output files
	xmlFile = tifFile;

	//Removing path from tifFile as it goes in src attribute of xml files
	int p = tifFile.rfind('\\');
	tifFile = tifFile.substr(p+1, tifFile.size() - p - 1);

	xmlFile.replace(xmlFile.size()-4, 4, ".xml");

	outFile.open(xmlFile.c_str());

	outFile << "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" << endl;
	outFile << "<!-- GEDI is developed at Language and Media Processing Laboratory, University of Maryland. -->" << endl;
	outFile << "<GEDI xmlns=\"http://lamp.cfar.umd.edu/GEDI\" version=\"1.0\">" << endl;
	outFile << "<USER name=\"Mudit\" date=\"May 21, 2008 8:25 AM\" />" << endl;
	outFile << "<DL_DOCUMENT src=\"" << tifFile << "\" docTag=\"xml\" NrOfPages=\"1\">" << endl;
	outFile << "<DL_PAGE gedi_type=\"DL_PAGE\" src=\"" << tifFile << "\" pageID=\"1\" width=\"" << width << "\" height=\"" << height << "\">" << endl;

	for(int i = 0; i < regions.size(); i++)
	{
		outFile <<  "<DL_ZONE gedi_type=\"MARKUP\" id=\"" << i << "\" col=\"1\" row=\"1\" width=\"1\" height=\"1\" polygon=\"";
		for(int j = 0; j < regions[i].edges.size(); j++)
		{
			outFile << "(" << regions[i].edges[j].dlGetFirstPoint().x << "," << regions[i].edges[j].dlGetFirstPoint().y << ");";
		}
		outFile << "\" contents=\"\" />" << endl;

	}

	outFile << "</DL_PAGE>" << endl;
	outFile << "</DL_DOCUMENT>" << endl;
	outFile << " </GEDI>" << endl;

	outFile.close();

}

void mergeRegions(vector <CRegionRep> & vRegions)
{
	FILE *fp;

	fp = fopen("sites.txt","w");

	//for(int i = 0; i < sample_pix; i++)
	//{
 //       fprintf(fp,"%f %f %d %d %d\n", sites[i].coord.x, sites[i].coord.y, sites[i].label, sites[i].refcnt, sites[i].sitenbr);
	//}

	//for(int i = 0; i < BPnbr; i++)
	//{
	//	fprintf(fp,"%u %u %u\n", bpx[i].xax, bpx[i].yax, bpx[i].label);
	//}

	//fprintf(fp, "\n Printing areas\n\n");
	//for(int i = 0; i < LABELnbr; i++)
	//{
	//	fprintf(fp,"%d %d\n",i,area[i]);
	//}
	fclose(fp);


}