/*
   $Id: defs.h,v 1.4 1999/10/15 12:36:21 kise Exp $
   $Date: 1999/10/15 12:36:21 $
   $Revision: 1.4 $
   $Author: kise $
*/

#ifndef DEFS_H_INCLUDED_
#define DEFS_H_INCLUDED_
#include "const.h"

typedef unsigned short Coordinate;
typedef unsigned short Label;
typedef unsigned int   NumPixel;
typedef unsigned int   Key;
typedef unsigned int   HashVal;
typedef double         Coord;

/* ٥ȥɽ¤
   the structure for a vector */
typedef struct{
  int x;
  int y;
} Vector;

/* ɽ¤
   the structure for a binary image */ 
typedef struct{
  char *image;
  Coordinate imax, jmax; /* width and height of the image */
} ImageData;

/* ǤΥ٥Ⱥɸɽ¤
   the structure for a pixel */
typedef struct{
  Label label;     /* label */
  Coordinate xax,yax; /* coordinates */
} BlackPixel;


/* ϢʬνſɸѤɽ¤
   the structure for a connected component */
typedef struct{
  Coordinate xc,yc; /* centroid of a min. bounding box */
  float area;           /* its area */
  float diameter;       /* its diameter */
} Component;

/* ܤϢʬδطɽ¤
   the structure for a neighboring relation
   between connected components(CC's)

         lab2
       -------
       |  x  |
       -------
    --- /
    | |/ angle
    |x|-----
    | |
    ---
    lab1
*/
typedef struct{
  float dist;       /* min. distance between CC's */
  float angle;      /* angle between CC's */
  Label lab1, lab2; /* labels of CC's */
  float area_ratio; /* area ratio of CC's (<1) */
  float dia_ratio;  /* diameter ratio of CC's (<1) */
  char regist;
} Neighborhood;

/* 󥯥ȥꥹȤι
   the structure for representing a list */
typedef struct tag_List{
  unsigned int number;
  struct tag_List *next;
} List;

typedef struct Point {
  float		x,y;
}Point;

/* 󥯥ȥꥹȤι
   the structure for representing a list */
typedef struct tag_List2{
  Point   coord;
  struct  tag_List2 *next;
} List2;

/* ܥΥդλ, ɤϢʬ֤Ǥ뤫ɽ¤
   the structure for a Voronoi edge */
typedef struct{
  int	sp,ep;
  Coordinate xs,xe,ys,ye;  /* + (xs,ys)
				\
				\  Voronoi edge
				\
				+ (xe,ye)
				*/
  Label lab1,lab2;  /* this Voronoi edge is between
			      CC of a label "lab1" and that of lab2 */
  unsigned short yn;
} LineSegment;

/* ܥΥդü
   the structure for a Voronoi point */
typedef struct node{
  int line;
  struct node *next;
} EndPoint;

/* ϢʬΥ٥Фϥåɽ
   the structure for a hash table for
   representing labels of CC's */
typedef struct hash {
  Label lab1;
  Label lab2;
  unsigned int entry;
  struct hash *next;
} HashTable;

/* ɽ¤
   the structure for a rectangle */
typedef struct{
    Coordinate is,ie,js,je;
} Rect;

/* ɤüǤɽ¤
   structure for an end of a seed */
typedef struct{
  unsigned int id; /* üλޤֹ
		      ID of an edge on an end of a seed
		      */
  Label label;     /* üϢʬֹ
		      label of a connected component
		      on an end of a seed
		      */
}Tip;

/* ɤɽ¤
   structure for a seed */
typedef struct{
  unsigned int *idlist;	/* ɤ˴ޤޤ neighbor ֹ
			   a list of id's of "neighbors"
			   included in a seed  */
  unsigned int current_idlist_size;
  Label *lablist;	/* ϢʬΥ٥
			   a list of labels of connected components */
  unsigned int current_lablist_size;
  Tip start;  /* ɤΥ饹ü
		 an end of a seed */
  Tip end;  /* ɤΥ饹Ǹü
	       an end of a seed */
  float mean_angle;
  float mean_dist;
  float mean_area;
  float mean_dia;
  float dispersion_angle;
  float dispersion_dist;
  int noe;		/* ɤ˴ޤޤ neighbor ο
			   the number of "neighbors" in a seed */
  int labnoe;             /* ɤ˴ޤޤ Ϣʬ
			     the number of connected components in a seed */
  char extend;            /* ɤĹɤ
			     whether or not a seed is extended */
} Seed;

struct Freenode {
    struct Freenode	*nextfree;
};

struct Freelist {
    struct Freenode	*head;
    int			nodesize;
};

/* structure used both for sites and for vertices */
struct Site {
  struct Point	coord;
  int		sitenbr;
  int		refcnt;
  Label	        label;
};

struct Edge {
    float		a,b,c;
    struct	Site 	*ep[2];
    struct	Site	*reg[2];
    int			edgenbr;
    unsigned int	lab1,lab2;
    float		dist;
};

struct Halfedge {
    struct Halfedge	*ELleft, *ELright;
    struct Edge		*ELedge;
    int			ELrefcnt;
    char		ELpm;
    struct Site		*vertex;
    float		ystar;
    struct Halfedge	*PQnext;
};


#endif /* DEFS_H_INCLUDED_ */
