/*
   $Id: function.h,v 1.5 1999/10/15 12:36:21 kise Exp $
   $Date: 1999/10/15 12:36:21 $
   $Revision: 1.5 $
   $Author: kise $
*/
#ifndef FUNCTION_H_INCLUDED_
#define FUNCTION_H_INCLUDED_
#include <stdio.h>
#include <stdlib.h>
#include "read_image.h"

/* 2dch.c */
void set_points( int, Coordinate, Coordinate );
int read_points( int );
float polygon_calc( int );
int ccw( Coord **, int, int, int );
int cmpl( const void *, const void * );
int cmph( const void *, const void * );
int make_chain( Coord **, int, int (*cmp)(const void*, const void*) );
int ch2d( int );
float area_calc( int );
Coord distance2( int, int );
Coord area1( int, int ); 
float diameter_calc( int );
void get_vertex( Point *, int );

/* bit_func.c */
int bit_get( ImageData *, Coordinate, Coordinate );
void bit_set( ImageData *, Coordinate, Coordinate, int );
int byte_pos ( char, int );
void frame( ImageData *, int, int );

/* ccomponent_stat.c */
void init_ccomponent_stat();
unsigned int peep_ccomponent_stat( Label );
void set_ccomponent_stat( Label, unsigned int );
void free_ccomponent_stat();

/* chtl.c */
void point_swap( int, int );
float signed_area( Point *, Point *, Point * );
int angular_compare( const void *, const void * );
void init_textline();
void convex_hull_of_line( int, int );
void set_gravity();
int locate( int, Point * );
void regist_inside();
void output_textline();

/* cline.c */
void analyze_cline( char **, int *, int * );

/* component.c */
void set_component( Coordinate, Coordinate );

/* dinfo.c */
void dparam();
void dnumber( int, int );
#ifdef TIME
void dtime();
#endif

/* edgelist.c */
void ELinitialize();
struct Halfedge *HEcreate( struct Edge *, int );
void ELinsert( struct Halfedge *, struct Halfedge * );
struct Halfedge *ELgethash( int );
struct Halfedge *ELleftbnd( struct Point * );
void ELdelete( struct Halfedge * );
struct Halfedge *ELright( struct Halfedge * );
struct Halfedge *ELleft( struct Halfedge * );
struct Site *leftreg( struct Halfedge * );
struct Site *rightreg( struct Halfedge * );

/* erase.c */
int start_pos ( int );
int end_pos ( int );
unsigned int Dh_ave( unsigned int *, int );
void hist();
int distinction( Label, Label, int );
void erase_endp( int );
void erase_aux();
void erase();
void init_u_int( unsigned int * );
void init_int( int * );
void ng_output();

/* fout.c */
void fout_sites( int, FILE * );
void fout_vor_edges( FILE * );

/* geometry.c */
void geominit();
struct Edge *bisect( struct Site *, struct Site * );
struct Site *intersect( struct Halfedge *, struct Halfedge * );
int right_of( struct Halfedge *, struct Point * );
void endpoint( struct Edge *, int, struct Site *, Coordinate, Coordinate );
float dist( struct Site *, struct Site * );
void makevertex( struct Site * );
void deref( struct Site * );
void ref( struct Site * );

/* hash.c */
HashVal hash1( Key );
HashVal hash2( Key );
void init_hash();
Key key( Label, Label );
int search( Label, Label );
void enter( Label, Label, unsigned int );

/* heap.c */
void PQinsert( struct Halfedge *, struct Site *, float );
void PQdelete( struct Halfedge * );
int PQbucket( struct Halfedge * );
int PQempty();
struct Point PQ_min();
struct Halfedge *PQextractmin();
void PQinitialize();

/* img_to_site.c */
void img_to_site( ImageData * );
void bf_edgelab_smpl( ImageData *, Coordinate, Coordinate, Label, char );
Vector next_point( ImageData *, Coordinate *, Coordinate *,
		   Vector *, Vector, Label );
Vector first_d( Vector );
Vector rot_d( Vector );
void edge_lab( Vector, Vector, Coordinate, Coordinate, Label );
void bpxset( Coordinate, Coordinate, Label );
int scomp( const void *, const void * );

/* label_func.c */
void lab_format( ImageData * );
Label lab_get( Coordinate, Coordinate );
void lab_set( Coordinate, Coordinate, Label );
void free_limg();

/* memory.c */
void freeinit( struct Freelist *, int );
char *getfree( struct Freelist * );
void makefree( struct Freenode *, struct Freelist * );
char *myalloc( unsigned );
char *myrealloc( void *, unsigned, unsigned int, size_t);

/* neighbor_graph.c */
void init_neighbor_graph();
void make_neighbor_graph();
List* peep_neighbor_graph( Label );
void inc_neighbor_graph( Label );
void dec_neighbor_graph( Label );
unsigned int number_neighbor_graph( Label );
int neighbor_compare( const void *, const void * );
void free_neighbor_graph();

/* output.c */
void in_frame( float *, float *, float, struct Edge *, int,
	       Coordinate, Coordinate );
void s_in_frame( float *, float *, float *, float *,
		 struct Edge *, Coordinate );
void e_in_frame( float *, float *, float *, float *,
		 struct Edge *, Coordinate, Coordinate );
void frameout( float *, float *, float *, float *,
	       int *, int *, struct Edge *, Coordinate, Coordinate );
void out_ep2( struct Edge *, struct Site *, Coordinate, Coordinate );

/* read_image.c */
void read_image( char *, ImageData * );
int tiff2imgd( char *, ImageData * );
int tiff_p( char * );
int ras2imgd( char *, ImageData * );
void swab_rashead( struct rasterfile * );

/* sampling_points.c */
int init_sampling_points();
int set_sampling_points( Label, float, float );
List2* get_sampling_points( Label );
void print_sp();

/* seed.c */
void init_seed();
void init_seed_aux( Seed * );
void init_idlist( unsigned int *, int, int );
void init_lablist( Label *, int, int );
void set_ratio();
void make_seed();
void resize_idlist( Seed * );
void resize_lablist( Seed * );
void tipset( unsigned int );
void element_calc( unsigned int );
void select_branch( unsigned int *, unsigned int, char );
int judge( unsigned int, unsigned int, char, int );
void extend_seed();
int compare( const void *, const void * );
void free_seed();
Label lablist_of_seed( int, int );
Label labnoe_of_seed( int );
int number_of_seed();
void set_textline();

/* sites.c */
struct Site *nextsite();

/* usage.c */
void usage();

/* voronoi.c */
void voronoi( Coordinate, Coordinate );
#endif /* FUNCTION_H_INCLUDED_ */
